pro MSP_BDP_Triage, image, NCOADD = ncoadd, triage_code, triage_message
;
;   assess if an image is good or bad by testing
;	for common failure characteristics
;
;	INPUTS:
;		image - pixel values in a 2D array (read in by calling program)
;		NCOADD - number of coadds in this image (or skip, if single)
;
;	OUTPUTS:
;		triage_code - integer representing binary encoded errors (see below)
;		triage_message - string that captures error types
;
;	based on triage.pro
;	triage history:
;   	20050608 DPC modified to pick up failed reads in CDS images (large negative mean counts)
;   	20050609 DPC modified to permit gaps in file numbers in directories
;   	20050610 DPC fixed empty file name problem
;   	20050611 DPC modified to allows not GUI interface (list input)
;   	20050707 DPC turned off quad test
;   	20050908 DPC added command line parameters to allow batch.pro running, simplified file selections
;	MSP_BDP_triage.pro history
;		20071031 DPC copied from triage, cut down to one image at a time
;		             added NCOADD flag to warn of the presence of coadded data (which changes limits)
;
if(N_ELEMENTS(NCOADD) eq 0) then ncoadd = 1
;
cflag0 = " /zero stddev in a quad (flag=0)/ "
cflag1 = " /LL quad mean very low (flag=1)/ "
cflag2 = " /LR quad mean very low (flag=2)/ "
cflag4 = " /UL quad mean very low (flag=4)/ "
cflag8 = " /UR quad mean very low (flag=8)/ "
cflag16 = " /2nd read failure (flag=16)/ "
cflag32 = " /1st read failure (flag=32)/ "
;
bad_flag0 = 0
bad_flag1 = 0
bad_flag2 = 0
bad_flag4 = 0
bad_flag8 = 0
bad_flag16 = 0
bad_flag32 = 0
;
;   compute image stats on each quadrant.
;
size_array = size(image)
xsize = size_array[1]
ysize = size_array[2]
;
;   compute size of band needed to skip ski jump top and bottom
;   (try about 9%)
;
yski = 0.09 * ysize
;
;   for each quadrants, compute image statistics - size computations needed to support
;     subframing
;
image_statistics, image[10:xsize/2-10,yski:ysize/2-10],MEAN=LL_mean,STDDEV=LL_stdev
image_statistics, image[10:xsize/2-10,ysize/2+10:ysize-yski],MEAN=UL_mean,STDDEV=UL_stdev
image_statistics, image[xsize/2+10:xsize-10,yski:ysize/2-10],MEAN=LR_mean,STDDEV=LR_stdev
image_statistics, image[xsize/2+10:xsize-10,ysize/2+10:ysize-yski],MEAN=UR_mean,STDDEV=UR_stdev
;
quad_means = [LL_mean, UL_mean, UR_mean, LR_mean]
quad_stdevs = [LL_stdev, UL_stdev, UR_stdev, LR_stdev]
;
;   tests, if any STDEVs are 0, set bad flag - (bad read condition)
;   if any MEANS are < 0.1 of the other means, set bad flag (partial bad read condition)
;
if (min(quad_stdevs) eq 0) then bad_flag0 = 1
;
;   test for any quad being really off compared to the others
;
;   off means being at least 30 sigma away from the mean of the other three
;     where sigma is the mean of the sigmas for each quad
;
sigma4 = total(quad_stdevs) / 4.0
criterion = 30.0
;
LL_test = UL_mean + UR_mean + LR_mean
LR_test = UL_mean + UR_mean + LL_mean
UL_test = LL_mean + UR_mean + LR_mean
UR_test = LL_mean + UL_mean + LR_mean
;
;     test for failed second read by looking for large, negative counts
;     if found, this is bad image
;
if (LL_mean lt -100*ncoadd) or (LR_mean lt -100*ncoadd) or $
	(UL_mean lt -100*ncoadd) or (UR_mean lt -100*ncoadd) then bad_flag16 = 1
;
;     test for failed first read by looking for large, positive counts
;
if (LL_mean gt ncoadd*25000L) or (LR_mean gt ncoadd*25000L) or $
		(UL_mean gt ncoadd*25000L) or (UR_mean gt ncoadd*25000L) then bad_flag32 = 1
;
;   if any bad flag set, add this file name to the bad files list, with flags set. Else,
;   add file name to good.files list.
;
net_bad_flag = bad_flag0+2*bad_flag1+4*bad_flag2+8*bad_flag4+16*bad_flag8+ $
				32*bad_flag16+64*bad_flag32
if (net_bad_flag eq 0) then begin
   ;printf, Unit_good, file_list(ifile)
   ;print, "Saving in good file list: ",file_list(ifile)
   triage_code = 0
   triage_message = ''
endif else begin
   if (bad_flag0 eq 1) then c0 = cflag0 else c0 = ''
   if (bad_flag1 eq 1) then c1 = cflag1 else c1 = ''
   if (bad_flag2 eq 1) then c2 = cflag2 else c2 = ''
   if (bad_flag4 eq 1) then c4 = cflag4 else c4 = ''
   if (bad_flag8 eq 1) then c8 = cflag8 else c8 = ''
   if (bad_flag16 eq 1) then c16 = cflag16 else c16 = ''
   if (bad_flag32 eq 1) then c32 = cflag32 else c32 = ''
   ;printf, Unit_bad, file_list(ifile), c0, c1, c2, c4, c8, c16, c32
   ;print, "Saving in bad file list: ",file_list(ifile)," Flag = ",bad_flag0/10.+bad_flag1+2*bad_flag2+4*bad_flag4+8*bad_flag8+16*bad_flag16+32*bad_flag32
   triage_code = net_bad_flag
   triage_message = STRING(format='(7a)',c0,c1,c2,c4,c8,c16,c32)
   ;
   stop
   ;
endelse
;
end
